@@ -36,7 +36,6 @@
 #ifndef HAVE_XCONVERTCASE
 #include "gdkkeysyms.h"
 #endif
-#include "gdkalias.h"
 
 typedef struct _GdkPredicate  GdkPredicate;
 
@@ -72,7 +71,6 @@ static const GDebugKey gdk_debug_keys[] = {
   {"xim",	    GDK_DEBUG_XIM},
   {"nograbs",       GDK_DEBUG_NOGRABS},
   {"colormap",	    GDK_DEBUG_COLORMAP},
-  {"gdkrgb",	    GDK_DEBUG_GDKRGB},
   {"gc",	    GDK_DEBUG_GC},
   {"pixmap",	    GDK_DEBUG_PIXMAP},
   {"image",	    GDK_DEBUG_IMAGE},
@@ -267,9 +265,6 @@ gdk_parse_args (int    *argc,
     }
   g_option_context_free (option_context);
 
-  if (_gdk_debug_flags && GDK_DEBUG_GDKRGB)
-    gdk_rgb_set_verbose (TRUE);
-
   GDK_NOTE (MISC, g_message ("progname: \"%s\"", g_get_prgname ()));
 }
 
@@ -338,9 +333,10 @@ gdk_display_open_default_libgtk_only (void)
   return display;
 }
 
-/*
- *--------------------------------------------------------------
- * gdk_init_check
+/**
+ * gdk_init_check:
+ * @argc: (inout):
+ * @argv: (array length=argc) (inout):
  *
  *   Initialize the library for use.
  *
@@ -359,7 +355,6 @@ gdk_display_open_default_libgtk_only (void)
  *
  *--------------------------------------------------------------
  */
-
 gboolean
 gdk_init_check (int    *argc,
 		char ***argv)
@@ -369,6 +364,12 @@ gdk_init_check (int    *argc,
   return gdk_display_open_default_libgtk_only () != NULL;
 }
 
+
+/**
+ * gdk_init:
+ * @argc: (inout):
+ * @argv: (array length=argc) (inout):
+ */
 void
 gdk_init (int *argc, char ***argv)
 {
@@ -380,31 +381,6 @@ gdk_init (int *argc, char ***argv)
     }
 }
 
-/*
- *--------------------------------------------------------------
- * gdk_exit
- *
- *   Restores the library to an un-itialized state and exits
- *   the program using the "exit" system call.
- *
- * Arguments:
- *   "errorcode" is the error value to pass to "exit".
- *
- * Results:
- *   Allocated structures are freed and the program exits
- *   cleanly.
- *
- * Side effects:
- *
- *--------------------------------------------------------------
- */
-
-void
-gdk_exit (gint errorcode)
-{
-  exit (errorcode);
-}
-
 void
 gdk_threads_enter (void)
 {
@@ -528,7 +504,7 @@ gdk_threads_dispatch_free (gpointer data)
  *            range btweeen #G_PRIORITY_DEFAULT_IDLE and #G_PRIORITY_HIGH_IDLE
  * @function: function to call
  * @data:     data to pass to @function
- * @notify:   function to call when the idle is removed, or %NULL
+ * @notify: (allow-none):   function to call when the idle is removed, or %NULL
  *
  * Adds a function to be called whenever there are no higher priority
  * events pending.  If the function returns %FALSE it is automatically
@@ -628,7 +604,7 @@ gdk_threads_add_idle (GSourceFunc    function,
  *             (1/1000ths of a second)
  * @function: function to call
  * @data:     data to pass to @function
- * @notify:   function to call when the timeout is removed, or %NULL
+ * @notify: (allow-none):   function to call when the timeout is removed, or %NULL
  *
  * Sets a function to be called at regular intervals holding the GDK lock,
  * with the given priority.  The function is called repeatedly until it 
@@ -734,7 +710,7 @@ gdk_threads_add_timeout (guint       interval,
  * @interval: the time between calls to the function, in seconds
  * @function: function to call
  * @data:     data to pass to @function
- * @notify:   function to call when the timeout is removed, or %NULL
+ * @notify: (allow-none):   function to call when the timeout is removed, or %NULL
  *
  * A variant of gdk_threads_add_timout_full() with second-granularity.
  * See g_timeout_add_seconds_full() for a discussion of why it is
@@ -806,5 +782,25 @@ gdk_set_program_class (const char *program_class)
   gdk_progclass = g_strdup (program_class);
 }
 
-#define __GDK_C__
-#include "gdkaliasdef.c"
+/**
+ * gdk_enable_multidevice:
+ *
+ * Enables multidevice support in GDK. This call must happen prior
+ * to gdk_display_open(), gtk_init(), gtk_init_with_args() or
+ * gtk_init_check() in order to take effect.
+ *
+ * Note that individual #GdkWindow<!-- -->s still need to explicitly
+ * enable multidevice awareness through gdk_window_set_support_multidevice().
+ *
+ * This function must be called before initializing GDK.
+ *
+ * Since: 3.0
+ **/
+void
+gdk_enable_multidevice (void)
+{
+  if (gdk_initialized)
+    return;
+
+  _gdk_enable_multidevice = TRUE;
+}
