@@ -32,11 +32,7 @@
 #include "gtkprogressbar.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
-#include "gtkalias.h"
 
-#define GTK_CELL_RENDERER_PROGRESS_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),                        \
-                                                                                     GTK_TYPE_CELL_RENDERER_PROGRESS, \
-                                                                                     GtkCellRendererProgressPrivate))
 
 enum
 {
@@ -232,7 +228,12 @@ gtk_cell_renderer_progress_class_init (GtkCellRendererProgressClass *klass)
 static void
 gtk_cell_renderer_progress_init (GtkCellRendererProgress *cellprogress)
 {
-  GtkCellRendererProgressPrivate *priv = GTK_CELL_RENDERER_PROGRESS_GET_PRIVATE (cellprogress);
+  GtkCellRendererProgressPrivate *priv;
+
+  cellprogress->priv = G_TYPE_INSTANCE_GET_PRIVATE (cellprogress,
+                                                    GTK_TYPE_CELL_RENDERER_PROGRESS,
+                                                    GtkCellRendererProgressPrivate);
+  priv = cellprogress->priv;
 
   priv->value = 0;
   priv->text = NULL;
@@ -246,8 +247,6 @@ gtk_cell_renderer_progress_init (GtkCellRendererProgress *cellprogress)
   priv->text_yalign = 0.5;
 
   priv->orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
-
-  cellprogress->priv = priv;
 }
 
 
@@ -358,8 +357,7 @@ recompute_label (GtkCellRendererProgress *cellprogress)
   if (priv->text)
     label = g_strdup (priv->text);
   else if (priv->pulse < 0)
-    /* do not translate the part before the | */
-    label = g_strdup_printf (Q_("progress bar label|%d %%"), priv->value);
+    label = g_strdup_printf (C_("progress bar label", "%d %%"), priv->value);
   else
     label = NULL;
  
@@ -397,10 +395,10 @@ gtk_cell_renderer_progress_set_pulse (GtkCellRendererProgress *cellprogress,
 
    if (pulse != priv->pulse)
      {
-       if (priv->pulse <= 0)
+       if (pulse <= 0)
          priv->offset = 0;
        else
-         priv->offset++;
+         priv->offset = pulse;
      }
 
    priv->pulse = pulse;
@@ -417,15 +415,18 @@ compute_dimensions (GtkCellRenderer *cell,
 {
   PangoRectangle logical_rect;
   PangoLayout *layout;
+  gint xpad, ypad;
   
   layout = gtk_widget_create_pango_layout (widget, text);
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
   
   if (width)
-    *width = logical_rect.width + cell->xpad * 2;
+    *width = logical_rect.width + xpad * 2;
   
   if (height)
-    *height = logical_rect.height + cell->ypad * 2;
+    *height = logical_rect.height + ypad * 2;
 
   g_object_unref (layout);
 }
@@ -446,7 +447,7 @@ gtk_cell_renderer_progress_get_size (GtkCellRenderer *cell,
 
   if (priv->min_w < 0)
     {
-      text = g_strdup_printf (Q_("progress bar label|%d %%"), 100);
+      text = g_strdup_printf (C_("progress bar label", "%d %%"), 100);
       compute_dimensions (cell, widget, text,
 			  &priv->min_w,
 			  &priv->min_h);
@@ -536,15 +537,17 @@ gtk_cell_renderer_progress_render (GtkCellRenderer *cell,
   PangoLayout *layout;
   PangoRectangle logical_rect;
   gint x, y, w, h, x_pos, y_pos, bar_position, bar_size, start, full_size;
+  gint xpad, ypad;
   GdkRectangle clip;
   gboolean is_rtl;
 
   is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
-  
-  x = cell_area->x + cell->xpad;
-  y = cell_area->y + cell->ypad;
-  w = cell_area->width - cell->xpad * 2;
-  h = cell_area->height - cell->ypad * 2;
+
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  x = cell_area->x + xpad;
+  y = cell_area->y + ypad;
+  w = cell_area->width - xpad * 2;
+  h = cell_area->height - ypad * 2;
 
   /* FIXME: GtkProgressBar draws the box with "trough" detail,
    * but some engines don't paint anything with that detail for
@@ -674,6 +677,3 @@ gtk_cell_renderer_progress_render (GtkCellRenderer *cell,
       g_object_unref (layout);
     }
 }
-
-#define __GTK_CELL_RENDERER_PROGRESS_C__
-#include "gtkaliasdef.c"
