@@ -24,7 +24,7 @@
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtktreeprivate.h"
-#include "gtkalias.h"
+
 
 static void gtk_cell_renderer_toggle_get_property  (GObject                    *object,
 						    guint                       param_id,
@@ -75,33 +75,36 @@ enum {
 
 static guint toggle_cell_signals[LAST_SIGNAL] = { 0 };
 
-#define GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE, GtkCellRendererTogglePrivate))
-
-typedef struct _GtkCellRendererTogglePrivate GtkCellRendererTogglePrivate;
-struct _GtkCellRendererTogglePrivate
+struct _GtkCellRendererTogglePriv
 {
   gint indicator_size;
 
+  guint active       : 1;
+  guint activatable  : 1;
   guint inconsistent : 1;
+  guint radio        : 1;
 };
 
 
 G_DEFINE_TYPE (GtkCellRendererToggle, gtk_cell_renderer_toggle, GTK_TYPE_CELL_RENDERER)
 
+
 static void
 gtk_cell_renderer_toggle_init (GtkCellRendererToggle *celltoggle)
 {
-  GtkCellRendererTogglePrivate *priv;
+  GtkCellRendererTogglePriv *priv;
 
-  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (celltoggle);
+  celltoggle->priv = G_TYPE_INSTANCE_GET_PRIVATE (celltoggle,
+                                                  GTK_TYPE_CELL_RENDERER_TOGGLE,
+                                                  GtkCellRendererTogglePriv);
+  priv = celltoggle->priv;
 
-  celltoggle->activatable = TRUE;
-  celltoggle->active = FALSE;
-  celltoggle->radio = FALSE;
+  priv->activatable = TRUE;
+  priv->active = FALSE;
+  priv->radio = FALSE;
 
-  GTK_CELL_RENDERER (celltoggle)->mode = GTK_CELL_RENDERER_MODE_ACTIVATABLE;
-  GTK_CELL_RENDERER (celltoggle)->xpad = 2;
-  GTK_CELL_RENDERER (celltoggle)->ypad = 2;
+  g_object_set (celltoggle, "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
+  gtk_cell_renderer_set_padding (GTK_CELL_RENDERER (celltoggle), 2, 2);
 
   priv->indicator_size = TOGGLE_WIDTH;
   priv->inconsistent = FALSE;
@@ -181,7 +184,7 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 		  G_TYPE_NONE, 1,
 		  G_TYPE_STRING);
 
-  g_type_class_add_private (object_class, sizeof (GtkCellRendererTogglePrivate));
+  g_type_class_add_private (object_class, sizeof (GtkCellRendererTogglePriv));
 }
 
 static void
@@ -191,23 +194,21 @@ gtk_cell_renderer_toggle_get_property (GObject     *object,
 				       GParamSpec  *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
-  GtkCellRendererTogglePrivate *priv;
+  GtkCellRendererTogglePriv *priv = celltoggle->priv;
 
-  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
-  
   switch (param_id)
     {
     case PROP_ACTIVE:
-      g_value_set_boolean (value, celltoggle->active);
+      g_value_set_boolean (value, priv->active);
       break;
     case PROP_INCONSISTENT:
       g_value_set_boolean (value, priv->inconsistent);
       break;
     case PROP_ACTIVATABLE:
-      g_value_set_boolean (value, celltoggle->activatable);
+      g_value_set_boolean (value, priv->activatable);
       break;
     case PROP_RADIO:
-      g_value_set_boolean (value, celltoggle->radio);
+      g_value_set_boolean (value, priv->radio);
       break;
     case PROP_INDICATOR_SIZE:
       g_value_set_int (value, priv->indicator_size);
@@ -226,23 +227,21 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
 				       GParamSpec   *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
-  GtkCellRendererTogglePrivate *priv;
-
-  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
+  GtkCellRendererTogglePriv *priv = celltoggle->priv;
 
   switch (param_id)
     {
     case PROP_ACTIVE:
-      celltoggle->active = g_value_get_boolean (value);
+      priv->active = g_value_get_boolean (value);
       break;
     case PROP_INCONSISTENT:
       priv->inconsistent = g_value_get_boolean (value);
       break;
     case PROP_ACTIVATABLE:
-      celltoggle->activatable = g_value_get_boolean (value);
+      priv->activatable = g_value_get_boolean (value);
       break;
     case PROP_RADIO:
-      celltoggle->radio = g_value_get_boolean (value);
+      priv->radio = g_value_get_boolean (value);
       break;
     case PROP_INDICATOR_SIZE:
       priv->indicator_size = g_value_get_int (value);
@@ -281,14 +280,16 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
 				   gint            *width,
 				   gint            *height)
 {
+  GtkCellRendererTogglePriv *priv;
   gint calc_width;
   gint calc_height;
-  GtkCellRendererTogglePrivate *priv;
+  gint xpad, ypad;
 
-  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (cell);
+  priv = GTK_CELL_RENDERER_TOGGLE (cell)->priv;
 
-  calc_width = (gint) cell->xpad * 2 + priv->indicator_size;
-  calc_height = (gint) cell->ypad * 2 + priv->indicator_size;
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  calc_width = xpad * 2 + priv->indicator_size;
+  calc_height = ypad * 2 + priv->indicator_size;
 
   if (width)
     *width = calc_width;
@@ -298,15 +299,19 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
 
   if (cell_area)
     {
+      gfloat xalign, yalign;
+
+      gtk_cell_renderer_get_alignment (cell, &xalign, &yalign);
+
       if (x_offset)
 	{
 	  *x_offset = ((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-		       (1.0 - cell->xalign) : cell->xalign) * (cell_area->width - calc_width);
+		       (1.0 - xalign) : xalign) * (cell_area->width - calc_width);
 	  *x_offset = MAX (*x_offset, 0);
 	}
       if (y_offset)
 	{
-	  *y_offset = cell->yalign * (cell_area->height - calc_height);
+	  *y_offset = yalign * (cell_area->height - calc_height);
 	  *y_offset = MAX (*y_offset, 0);
 	}
     }
@@ -326,20 +331,20 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
 				 GdkRectangle         *expose_area,
 				 GtkCellRendererState  flags)
 {
-  GtkCellRendererToggle *celltoggle = (GtkCellRendererToggle *) cell;
-  GtkCellRendererTogglePrivate *priv;
+  GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (cell);
+  GtkCellRendererTogglePriv *priv = celltoggle->priv;
   gint width, height;
   gint x_offset, y_offset;
+  gint xpad, ypad;
   GtkShadowType shadow;
   GtkStateType state = 0;
 
-  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (cell);
-
   gtk_cell_renderer_toggle_get_size (cell, widget, cell_area,
 				     &x_offset, &y_offset,
 				     &width, &height);
-  width -= cell->xpad*2;
-  height -= cell->ypad*2;
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  width -= xpad * 2;
+  height -= ypad * 2;
 
   if (width <= 0 || height <= 0)
     return;
@@ -347,35 +352,36 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
   if (priv->inconsistent)
     shadow = GTK_SHADOW_ETCHED_IN;
   else
-    shadow = celltoggle->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
+    shadow = priv->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
 
-  if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE || !cell->sensitive)
+  if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE ||
+      !gtk_cell_renderer_get_sensitive (cell))
     {
       state = GTK_STATE_INSENSITIVE;
     }
   else if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
     {
-      if (GTK_WIDGET_HAS_FOCUS (widget))
+      if (gtk_widget_has_focus (widget))
 	state = GTK_STATE_SELECTED;
       else
 	state = GTK_STATE_ACTIVE;
     }
   else
     {
-      if (celltoggle->activatable)
+      if (priv->activatable)
         state = GTK_STATE_NORMAL;
       else
         state = GTK_STATE_INSENSITIVE;
     }
 
-  if (celltoggle->radio)
+  if (priv->radio)
     {
       gtk_paint_option (widget->style,
                         window,
                         state, shadow,
                         expose_area, widget, "cellradio",
-                        cell_area->x + x_offset + cell->xpad,
-                        cell_area->y + y_offset + cell->ypad,
+                        cell_area->x + x_offset + xpad,
+                        cell_area->y + y_offset + ypad,
                         width, height);
     }
   else
@@ -384,8 +390,8 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
                        window,
                        state, shadow,
                        expose_area, widget, "cellcheck",
-                       cell_area->x + x_offset + cell->xpad,
-                       cell_area->y + y_offset + cell->ypad,
+                       cell_area->x + x_offset + xpad,
+                       cell_area->y + y_offset + ypad,
                        width, height);
     }
 }
@@ -399,10 +405,13 @@ gtk_cell_renderer_toggle_activate (GtkCellRenderer      *cell,
 				   GdkRectangle         *cell_area,
 				   GtkCellRendererState  flags)
 {
+  GtkCellRendererTogglePriv *priv;
   GtkCellRendererToggle *celltoggle;
-  
+
   celltoggle = GTK_CELL_RENDERER_TOGGLE (cell);
-  if (celltoggle->activatable)
+  priv = celltoggle->priv;
+
+  if (priv->activatable)
     {
       g_signal_emit (cell, toggle_cell_signals[TOGGLED], 0, path);
       return TRUE;
@@ -428,9 +437,13 @@ void
 gtk_cell_renderer_toggle_set_radio (GtkCellRendererToggle *toggle,
 				    gboolean               radio)
 {
+  GtkCellRendererTogglePriv *priv;
+
   g_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle));
 
-  toggle->radio = radio;
+  priv = toggle->priv;
+
+  priv->radio = radio;
 }
 
 /**
@@ -446,7 +459,7 @@ gtk_cell_renderer_toggle_get_radio (GtkCellRendererToggle *toggle)
 {
   g_return_val_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle), FALSE);
 
-  return toggle->radio;
+  return toggle->priv->radio;
 }
 
 /**
@@ -463,7 +476,7 @@ gtk_cell_renderer_toggle_get_active (GtkCellRendererToggle *toggle)
 {
   g_return_val_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle), FALSE);
 
-  return toggle->active;
+  return toggle->priv->active;
 }
 
 /**
@@ -498,7 +511,7 @@ gtk_cell_renderer_toggle_get_activatable (GtkCellRendererToggle *toggle)
 {
   g_return_val_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle), FALSE);
 
-  return toggle->activatable;
+  return toggle->priv->activatable;
 }
 
 /**
@@ -514,14 +527,15 @@ void
 gtk_cell_renderer_toggle_set_activatable (GtkCellRendererToggle *toggle,
                                           gboolean               setting)
 {
+  GtkCellRendererTogglePriv *priv;
+
   g_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle));
 
-  if (toggle->activatable != setting)
+  priv = toggle->priv;
+
+  if (priv->activatable != setting)
     {
-      toggle->activatable = setting ? TRUE : FALSE;
+      priv->activatable = setting ? TRUE : FALSE;
       g_object_notify (G_OBJECT (toggle), "activatable");
     }
 }
-
-#define __GTK_CELL_RENDERER_TOGGLE_C__
-#include "gtkaliasdef.c"
