@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -92,18 +92,15 @@ typedef enum
 
 
 typedef struct _GtkDialog        GtkDialog;
+typedef struct _GtkDialogPriv    GtkDialogPriv;
 typedef struct _GtkDialogClass   GtkDialogClass;
 
 struct _GtkDialog
 {
   GtkWindow window;
 
-  /*< public >*/
-  GtkWidget *GSEAL (vbox);
-  GtkWidget *GSEAL (action_area);
-
   /*< private >*/
-  GtkWidget *GSEAL (separator);
+  GtkDialogPriv *priv;
 };
 
 struct _GtkDialogClass
@@ -148,6 +145,8 @@ void gtk_dialog_set_response_sensitive (GtkDialog *dialog,
                                         gboolean   setting);
 void gtk_dialog_set_default_response   (GtkDialog *dialog,
                                         gint       response_id);
+GtkWidget* gtk_dialog_get_widget_for_response (GtkDialog *dialog,
+                                               gint       response_id);
 gint gtk_dialog_get_response_for_widget (GtkDialog *dialog,
 					 GtkWidget *widget);
 
