@@ -25,7 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -45,24 +45,15 @@ G_BEGIN_DECLS
 
 
 typedef struct _GtkStatusbar      GtkStatusbar;
+typedef struct _GtkStatusbarPriv  GtkStatusbarPriv;
 typedef struct _GtkStatusbarClass GtkStatusbarClass;
 
 struct _GtkStatusbar
 {
   GtkHBox parent_widget;
 
-  GtkWidget *GSEAL (frame);
-  GtkWidget *GSEAL (label);
-
-  GSList *GSEAL (messages);
-  GSList *GSEAL (keys);
-
-  guint GSEAL (seq_context_id);
-  guint GSEAL (seq_message_id);
-
-  GdkWindow *GSEAL (grip_window);
-
-  guint GSEAL (has_resize_grip) : 1;
+  /* <private> */
+  GtkStatusbarPriv *priv;
 };
 
 struct _GtkStatusbarClass
@@ -102,6 +93,9 @@ void       gtk_statusbar_pop          	(GtkStatusbar *statusbar,
 void       gtk_statusbar_remove        	(GtkStatusbar *statusbar,
 					 guint	       context_id,
 					 guint         message_id);
+void       gtk_statusbar_remove_all    	(GtkStatusbar *statusbar,
+					 guint	       context_id);
+					 
 
 void     gtk_statusbar_set_has_resize_grip (GtkStatusbar *statusbar,
 					    gboolean      setting);
