@@ -22,8 +22,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#undef GTK_DISABLE_DEPRECATED
-
 #include <gtk/gtk.h>
 
 #include "gailtoplevel.h"
@@ -325,9 +323,10 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
 static gboolean
 is_attached_menu_window (GtkWidget *widget)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
   gboolean ret = FALSE;
 
+  child = gtk_bin_get_child (GTK_BIN (widget));
   if (GTK_IS_MENU (child))
     {
       GtkWidget *attach;
@@ -335,7 +334,6 @@ is_attached_menu_window (GtkWidget *widget)
       attach = gtk_menu_get_attach_widget (GTK_MENU (child));
       /* Allow for menu belonging to the Panel Menu, which is a GtkButton */
       if (GTK_IS_MENU_ITEM (attach) ||
-          GTK_IS_OPTION_MENU (attach) ||
           GTK_IS_BUTTON (attach))
         ret = TRUE;
     }
@@ -345,19 +343,21 @@ is_attached_menu_window (GtkWidget *widget)
 static gboolean
 is_combo_window (GtkWidget *widget)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
   AtkObject *obj;
   GtkAccessible *accessible;
 
+  child = gtk_bin_get_child (GTK_BIN (widget));
+
   if (!GTK_IS_EVENT_BOX (child))
     return FALSE;
 
-  child = GTK_BIN (child)->child;
+  child = gtk_bin_get_child (GTK_BIN (child));
 
   if (!GTK_IS_FRAME (child))
     return FALSE;
 
-  child = GTK_BIN (child)->child;
+  child = gtk_bin_get_child (GTK_BIN (child));
 
   if (!GTK_IS_SCROLLED_WINDOW (child))
     return FALSE;
@@ -365,8 +365,6 @@ is_combo_window (GtkWidget *widget)
   obj = gtk_widget_get_accessible (child);
   obj = atk_object_get_parent (obj);
   accessible = GTK_ACCESSIBLE (obj);
-  if (GTK_IS_COMBO (accessible->widget))
-    return TRUE;
 
   return  FALSE;
 }
